///

#include "SmgStateMachine.hpp"

SmgStateMachine::SmgStateMachine(SmartGridModel &model)
    : m_model{model}
{
}


void SmgStateMachine::run()
{
    if (m_state_machine_is_running)
    {
        switch (m_main_state)
        {
        case MainState::Simulation:
            simulation_state();
            break;
        case MainState::ManualControl:
            manual_state();
            break;
        }
    }
}


void SmgStateMachine::simulation_state()
{
    switch (m_sim_state)
    {
    case SimState::Entry:
        m_sim_state = SimState::Do;
        break;
    case SimState::Do:
            m_model.next_hour();
            if (false) // some external condition
            {
                m_sim_state = SimState::Exit;
            }
        break;
    case SimState::Exit:
        m_sim_state = SimState::Entry;
        m_main_state = MainState::ManualControl;
        m_manual_state = ManualState::Entry;
        break;
    }
}

void SmgStateMachine::manual_state()
{
    switch (m_manual_state)
    {
    case ManualState::Entry:
        m_manual_state = ManualState::Do;
        break;
    case ManualState::Do:

        // apply changes
        m_model.update_states();
        if (false) // some external condition
        {
            m_manual_state = ManualState::Exit;
        }
        break;
    case ManualState::Exit:
        m_manual_state = ManualState::Entry;
        m_main_state = MainState::Simulation;
        m_sim_state = SimState::Entry;
        break;
    }
}

bool SmgStateMachine::is_sim_mode_enabled()
{
    return MainState::Simulation == m_main_state;
}

void SmgStateMachine::set_sim_mode_enabled(bool enable)
{
    if (enable)
    {
        if (m_main_state == MainState::ManualControl)
        {
            m_manual_state = ManualState::Exit;
        }
    }
    else
    {
        if (m_main_state == MainState::Simulation)
        {
            m_sim_state = SimState::Exit;
        }
    }
}