/// @file   Protocol.cpp
///

#include <ProtocolSgm.hpp>
#include <com/Protocol.hpp>


ProtocolSgm::ProtocolSgm(std::string const& address)
    : m_com_addr{address}
{
    m_sending = std::thread(&ProtocolSgm::sending, this);
    m_receiving = std::thread(&ProtocolSgm::receiving, this);
}

void ProtocolSgm::sending()
{
    if (m_send_queue->empty() == true)
    {
        std::this_thread::sleep_for(std::chrono::milliseconds(1));
    }
    else
    {
        auto cmd = m_send_queue->back();
        ConnectSocket connect{m_com_addr, sgm_com_port};
        ProtocolSimple prot{connect};

        prot.send_message("", cmd.cmd);
        prot.send_message("", cmd.val);
        prot.send_message("", cmd.ack);

        std::string ack;
        prot.recv_message(ack); // done
        if (ack != cmd.ack)
        {
            // something went wrong
        }
        else
        {
            m_send_queue->pop_back();
        }
    }
}

void ProtocolSgm::receiving()
{
    ServerSocket server{sgm_com_port};
    bool continue_rece {true};
    while (true)
    {
        DataSocket sock = server.accept();
        ProtocolSimple prot{sock};

        CommandSgm cmd {};
        prot.recv_message(cmd.cmd);
        prot.recv_message(cmd.val);
        prot.recv_message(cmd.ack);

        m_recv_queue->push_back(cmd);

        prot.send_message(m_com_addr, cmd.ack);
    }
}


void ProtocolSgm::send_cmd(CommandSgm const& cmd)
{
    m_send_queue->push_back(cmd);
}