/// @file   Socket.hpp
/// Whats is going on

#ifndef SOCKET_HPP
#define SOCKET_HPP

#include <string>


class Socket
{
public:
    Socket(Socket&& move);
    Socket& operator=(Socket&& move);
    virtual ~Socket();
    void close();
    void swap(Socket& other);
protected:
    Socket(int fd);
    int get_socket() const;
private:
    Socket(Socket const&) = delete;
    Socket& operator=(Socket const&) = delete;
    int m_fd;
};


class DataSocket : public Socket
{
public:
    DataSocket(int fd);
    std::size_t get_data(char* data, std::size_t size);
    void put_data(char const* data, std::size_t size);
    void end_transmission();
};


class ConnectSocket : public DataSocket
{
public:
    ConnectSocket(std::string const& host, int port);
};


class ServerSocket : public Socket
{
    static constexpr int max_connections {5};
public:
    ServerSocket(int port);
    DataSocket accept();
};


#endif