/// @brief Test Commands to Arduino with I2C
///

// to open i2c device
#include <fcntl.h>

// select and interact with device
#include <sys/ioctl.h>

// Definition of I2C_SLAVE
#include <linux/i2c-dev.h>

// Nice and easy logging
#include <spdlog/spdlog.h>

#include <unistd.h>
#include<vector>
#include<array>

/// @brief Device name of the I2C interface on the Pi
///
const char* i2c_device_name = "/dev/i2c-1";
constexpr uint8_t arduino_address = 0x14;

void write_to_node(int fd, std::vector<uint8_t> data)
{
    int buffer_size = data.size();
    int bytes_written = write(fd, data.data(), buffer_size);
    if (bytes_written != buffer_size)
    {
        spdlog::error("Failed to write buffer over I2C");
        spdlog::info("Bytes written {}", bytes_written);
    }
}


int main(int argc, char* argv[])
{
    int fd_i2c = open(i2c_device_name, O_RDWR);
    if (fd_i2c < 0)
    {
        spdlog::error("Unable to open I2C device");
        return 1;
    }
    int device_selected = ioctl(fd_i2c, I2C_SLAVE, arduino_address);
    if (device_selected < 0)
    {
        spdlog::error("Unable to select I2C device");
        return 1;
    }
    if (argc == 3)
    {
        uint8_t arg1 = atoi(argv[1]);
        uint8_t arg2 = atoi(argv[2]);

        write_to_node(fd_i2c, {arg1, arg2});
        spdlog::info("Bytes written: {} {}", arg1, arg2);
    }
    else
    {
        spdlog::error("Number of input arguments is not correct!");
        spdlog::error("Usage: val1 val2");
        for (int i = 0; i < argc; ++i)
        {
            spdlog::info("arg{}: {}", i, argv[i]);
        }
    }
    close(fd_i2c);
    return 0;
}
